﻿using System;
using System.IO;
using System.Xml;
using System.Collections.Generic;

using Prova1.Model;

namespace Prova1.Persistence
{
    public class XmlExamReader : IExamReader
    {
        private readonly XmlDocument _xmlDocument;

        public XmlExamReader(TextReader reader)
        {
            _xmlDocument = new XmlDocument();
            _xmlDocument.Load(reader);
            reader.Close();
        }

        public IList<Question> ReadQuestions()
        {
            List<Question> questions = new List<Question>();
            foreach (XmlElement questionElement in _xmlDocument.SelectNodes("/exam/*"))
            {
                Question question = ReadQuestion(questionElement);
                questions.Add(question);
            }
            return questions;
        }

        private Question ReadQuestion(XmlElement questionElement)
        {
            switch (questionElement.Name)
            {
                case "multipleChoice":
                    return ReadMultipleChoiceQuestion(questionElement);
                case "fillInTheBlanks":
                    return ReadFillInTheBlanksQuestion(questionElement);
                default:
                    throw new ApplicationException("Invalid question type");
            }
        }

        private MultipleChoiceQuestion ReadMultipleChoiceQuestion(XmlElement questionElement)
        {
            List<Answer> answers = new List<Answer>();
            foreach (XmlElement answerElement in questionElement.SelectNodes("answer"))
            {
                Answer answer = new Answer();
                answer.Points = XmlConvert.ToDouble(answerElement.GetAttribute("points"));
                answer.Text = answerElement.InnerText;
                answers.Add(answer);
            }
            MultipleChoiceQuestion question = new MultipleChoiceQuestion(answers);
            ReadCommons(question, questionElement);
            return question;
        }

        private FillInTheBlanksQuestion ReadFillInTheBlanksQuestion(XmlElement questionElement)
        {
            List<IPart> parts = new List<IPart>();
            foreach (XmlElement partElement in questionElement.SelectNodes("toComplete/*"))
            {
                IPart part = ReadFillInTheBlanksPart(partElement);
                parts.Add(part);
            }
            FillInTheBlanksQuestion question = new FillInTheBlanksQuestion(parts);
            ReadCommons(question, questionElement);
            return question;
        }

        private IPart ReadFillInTheBlanksPart(XmlElement partElement)
        {
            switch (partElement.Name)
            {
                case "text":
                    return new TextPart() { Text = partElement.InnerText };
                case "blank":
                    return new EditablePart()
                        {
                            ExpectedText = partElement.InnerText,
                            RightAnswerPoints = XmlConvert.ToDouble(partElement.GetAttribute("right")),
                            WrongAnswerPoints = XmlConvert.ToDouble(partElement.GetAttribute("wrong")),
                        };
                default:
                    throw new ApplicationException("Invalid part type");
            }
        }

        private void ReadCommons(Question question, XmlElement questionElement)
        {
            question.Id = questionElement.GetAttribute("id");

            XmlElement textElement = (XmlElement) questionElement.SelectSingleNode("text");
            question.Text = textElement.InnerText;
        }
    }
}
